#!/usr/bin/env python

import os
import pygtk
pygtk.require('2.0')
import gtk
import gobject

from gmusicapi import Mobileclient
from music_player import MusicPlayer
from widget import (SourcePane, ArtistPane, AlbumPane, SongPane,
                    get_player_control_toolbar, MainWindow, Login,
                    SongListStore)

api = Mobileclient()

gobject.threads_init()
from lastfm import AudioScrobbler

class Player(object):

    def __init__(self):
        self.window = MainWindow(gtk.WINDOW_TOPLEVEL)
        path = os.path.expanduser("~/.config/google_music_player")
        if not os.path.exists(path):
            message = Login(parent=self.window)
            message.run()
        path = os.path.expanduser("~/.config/google_music_player")
        while True:
            with open(path, "r") as fd:
                auth = fd.readlines()
                login, password = auth[:2]
                logged = api.login(
                    login.strip(),
                    password.strip(),
                    Mobileclient.FROM_MAC_ADDRESS)
                if logged:
                    last_fm = None
                    if len(auth) > 2:
                        try:
                            last_fm = AudioScrobbler()
                            last_fm.login(*auth[2:])
                        except Exception as e:
                            print e
                    else:
                        print "No Last"
                    break
                else:
                    message = Login(parent=self.window)
                    message.run()

        self.t = None # The music thread
        # Create a new window

        self.window.connect("delete_event", self.delete_event)

        # create the liststores (artist, album, songs)
        self.create_stores()
        self.refresh()
        self.create_base_panes()
        self.create_autocomplete()
        self.create_label_image()
        self.create_main_ui()
        self.create_right_click_menu()
        self.source_pane.connect("row-activated", self.expand)
        self.treeview.connect("row-activated", self.on_clicked)
        self.treeview.connect('button-press-event' , self.on_right_click)
        self.album_pane.get_selection().connect("changed", self.filter_album)
        self.artist_pane.get_selection().connect("changed", self.filter_artist)

        self.t = MusicPlayer(api, self, self.treeview, lastfm=last_fm)

    def create_base_panes(self):
        self.source_pane = SourcePane(self.source_store)
        self.source_scrolled_window = gtk.ScrolledWindow()
        self.source_scrolled_window.set_policy(
            gtk.POLICY_NEVER,
            gtk.POLICY_ALWAYS)
        self.source_scrolled_window.add(self.source_pane)


        self.artist_pane = ArtistPane(self.artist_store)
        self.artist_scrolled_window = gtk.ScrolledWindow()
        self.artist_scrolled_window.add(self.artist_pane)

        self.album_pane = AlbumPane(self.album_store)
        self.album_scrolled_window = gtk.ScrolledWindow()

        self.album_scrolled_window.add(self.album_pane)

        self.treeview = SongPane(self.liststore)
        self.scrolled_window = gtk.ScrolledWindow()
        self.scrolled_window.add(self.treeview)

    def create_right_click_menu(self):
        self.right_menu = gtk.Menu()
        like = gtk.MenuItem("Love")
        dislike = gtk.MenuItem("Dislike")
        clear = gtk.MenuItem("Clear rating")
        self.right_menu.append(like)
        self.right_menu.append(dislike)
        self.right_menu.append(clear)
        like.connect("activate", self.on_like, "like")
        dislike.connect("activate", self.on_like, "dislike")
        clear.connect("activate", self.on_like, "clear")
        like.show()
        dislike.show()
        clear.show()

    def create_autocomplete(self):
        self.completion_store = gtk.ListStore(str, str, str)
        completion = gtk.EntryCompletion()
        completion.set_model(self.completion_store)
        completion.set_text_column(0)
        self.search = gtk.Entry()
        self.search.set_completion(completion)
        self.search.connect("key-press-event", self.autocomplete)
        self.search.connect("activate", self.get_selection)

    def create_label_image(self):
        self.song_label = gtk.Label("")
        self.album_pic = gtk.Image()

    def create_main_ui(self):
        self.box = gtk.VBox()
        self.menubox = gtk.HBox()
        self.filters_box = gtk.HBox()
        self.main_box = gtk.HBox()
        self.right_box = gtk.VBox()

        # create the top menu bar
        self.menubox.pack_start(get_player_control_toolbar(self))
        self.menubox.pack_start(self.song_label)
        self.menubox.pack_end(self.search)

        # create the filter box
        self.filters_box.pack_start(self.artist_scrolled_window)
        self.filters_box.pack_start(self.album_scrolled_window)

        # create the right box
        self.right_box.pack_start(self.source_scrolled_window)
        self.right_box.pack_start(self.album_pic, expand=False)

        # create the vbox to hold filters and songs
        self.vbox = gtk.VBox()
        self.vbox.pack_start(self.filters_box)
        self.vbox.pack_start(self.scrolled_window)

        # add the rigth box to the main box
        self.main_box.pack_start(self.right_box, expand=False)
        # add the filters and songs to the main box
        self.main_box.pack_start(self.vbox)

        # add the menu to the window
        self.box.pack_start(self.menubox, expand=False)
        # add main box to the window
        self.box.add(self.main_box)

        self.window.add(self.box)
        self.window.show_all()

    def get_selection(self, widget, *args):
        text = widget.get_text()
        obj_type = None
        tracks = None
        for row in self.completion_store:
            if row[0] == text:
                obj_id, obj_type = row[1], row[2]
                break
        if obj_type == "artist":
            tracks = api.get_artist_info(
                obj_id, max_top_tracks=20)['topTracks']
        elif obj_type == "album":
            tracks = api.get_album_info(
                obj_id
            )['tracks']
        if tracks:
            for entry in tracks:
                entry["id"] = entry["storeId"]
            self.refresh(tracks)

    def autocomplete(self, widget, key):
        string = u"{}{}".format(self.search.get_text(), key.string)
        results = api.search(string, max_results=10)
        self.completion_store.clear()
        if results.get('artist_hits'):
            for artist in results.get('artist_hits'):

                self.completion_store.append([artist["artist"]["name"],
                                             artist["artist"]["artistId"],
                                              "artist"])
        if results.get('album_hits'):
            for album in results.get('album_hits'):

                self.completion_store.append([album["album"]["name"],
                                              album["album"]["albumId"],
                                              "album"])
        # print results.get('playlist_hits')
        # print results.get('song_hits')[0]

    # close the window and quit
    def delete_event(self, widget, event, data=None):
        gtk.main_quit()
        return False

    def create_stores(self):
        self.liststore = SongListStore(str, str, str, str, str, str, str)
        self.album_store = gtk.ListStore(str)
        self.artist_store = gtk.ListStore(str)
        self.source_store = gtk.TreeStore(str)
        for elem in ["library", "playlists", "radios"]:
            self.source_store.append(None, [elem])

    def filter_songs(self, filter_key=None, filter_term=None):
        for song in self.songs:
            if not filter_key or song[filter_key] == filter_term:
                rating = song.get('rating')
                if rating == '5':
                    rating = '+1'
                elif rating == '1':
                    rating = '-1'
                else:
                    rating = None
                self.liststore.append(
                    [song["artist"],
                     song['album'],
                     song["title"],
                     song['id'],
                     song['albumArtRef'][0]['url'],
                     song["storeId"],
                     rating])

    def refresh(self, songs=None):
        if songs is None:
            self.songs = api.get_all_songs()

        else:
            self.songs = songs
        for store in [self.liststore, self.album_store, self.artist_store]:
            store.clear()
        for album in sorted(set([song['album'] for song in self.songs])):
            self.album_store.append([album])

        for artist in sorted(set([song['artist'] for song in self.songs])):
            self.artist_store.append([artist])
        self.songs = sorted(self.songs, key=lambda k: k['title'])
        self.filter_songs()


    def filter_album(self, selection):
        model, index = selection.get_selected_rows()
        if len(index) == 0:
            return
        index = index[0][0]
        album = self.album_store[index][0]
        self.liststore.clear()
        self.filter_songs('album', album)

    def filter_artist(self, selection):
        model, index = selection.get_selected_rows()
        index = index[0][0]
        artist = self.artist_store[index][0]
        self.liststore.clear()
        album_songs = []
        self.filter_songs('artist', artist)
        for song in self.songs:
            if song['artist'] == artist:
                album_songs.append(song['album'])

        self.album_store.clear()
        for album in set(album_songs):
            self.album_store.append([album])

    def on_clicked(self, widget, index, item):
        index = index[0]
        self.play()

    def on_right_click(self, widget, event):
        if event.button == 3:
            self.right_menu.popup(None, None, None, event.button, event.time)

    def on_like(self, widget, event):

        selection = self.treeview.get_selection()
        index = selection.get_selected_rows()[1][0][0]
        self.liststore.set_index(index)
        song = api.get_track_info(
            unicode(self.liststore[self.liststore.get_index()][5])
        )
        rating = ''
        if event == 'like':
            song['rating'], rating = '5', '+1'
        elif event == 'dislike':
            # dislike
            song['rating'], rating = '1', '-1'
        elif event == "clear":
            song['rating'], rating = '0', None
        api.change_song_metadata(song)
        self.liststore[index][6] = rating # update the ui

    def play(self):
        selection = self.treeview.get_selection()
        try:
            index = selection.get_selected_rows()[1][0][0]
        except IndexError:
            index = 0
        self.liststore.set_index(index)
        self.t.play()

    def get_index(self):
        return self.liststore.get_index()

    def set_index(self, index):
        return self.liststore.set_index(index)

    def previous(self):
        if self.get_index() > 0:
            self.set_index(self.get_index() - 1)
            self.t.play()

    def next(self):
        if self.get_index() +1 < self.liststore:
            self.set_index(self.get_index() + 1)
            self.t.play()


    def pause(self):
        if self.t:
            self.t.p.pause()

    def expand(self, widget, index, item):
        index = index
        it = self.source_store.get_iter(index)
        menu = self.source_store.get(it, 0)
        if len(index) == 1:
            if menu[0] == "playlists":
                self.playlists = api.get_all_user_playlist_contents()
                self.playlists = sorted(self.playlists, key=lambda k: k['name'])
                for playlist in self.playlists:
                    self.source_store.append(
                        self.source_store.get_iter(1),
                        [playlist["name"]])
            if menu[0]== "radios":
                self.radios = api.get_all_stations()
                self.radios = sorted(self.radios, key=lambda k: k['name'])
                for radio in self.radios:
                    self.source_store.append(
                        self.source_store.get_iter(2),
                        [radio["name"]])
        else:
            source_type = self.source_store.get_iter(index[0])
            if self.source_store.get(source_type, 0)[0] == "playlists":
                playlist = (
                    item for item in self.playlists
                    if item["name"] == menu[0]
                ).next()
                for entry in playlist['tracks']:
                    try:
                        entry["track"]["id"] = entry["trackId"]
                    except KeyError:
                        pass
                tracks = []
                for entry in playlist['tracks']:
                    if entry.get("track"):
                        tracks.append(entry['track'])
                self.refresh(tracks)
            elif self.source_store.get(source_type, 0)[0] == "radios":
                radio = (
                    item for item in self.radios
                    if item["name"] == menu[0]
                ).next()
                tracks = []
                for track in api.get_station_tracks(radio['id']):
                    track['id'] = track['nid']
                    tracks.append(track)
                self.refresh(tracks)
def main():
    gtk.main()

if __name__ == "__main__":

    player = Player()
    main()



import gtk
import os
from gettext import gettext as _

class SongListStore(gtk.ListStore):

    def __init__(self, *args, **kwargs):
        super(SongListStore, self).__init__(*args, **kwargs)
        self.index = 0

    def clear(self, *args, **kwargs):
        super(SongListStore, self).clear(*args, **kwargs)
        self.index = 0

    def get_index(self):
        return self.index

    def set_index(self, index):
        self.index = index
        return self.index

def get_player_control_toolbar(win):
    """Return a player control toolbar
    """
    tb = gtk.Toolbar()
    tb.set_style(gtk.TOOLBAR_ICONS)
    for text, tooltip, stock, callback in (

            (_("Previous"), _("Previous"),
             gtk.STOCK_MEDIA_PREVIOUS, lambda b: win.previous()),

            (_("Play"), _("Play"),
             gtk.STOCK_MEDIA_PLAY,lambda b: win.play()),

            (_("Pause"), _("Pause"),
             gtk.STOCK_MEDIA_PAUSE, lambda b: win.pause()),

            (_("Next"), _("Next"),
             gtk.STOCK_MEDIA_NEXT, lambda b: win.next()),

            (_("Refresh"),_("Refresh"),
             gtk.STOCK_REFRESH, lambda b: win.refresh())

            ):
        b=gtk.ToolButton(stock)
        b.set_tooltip_text(tooltip)
        b.connect("clicked", callback)
        tb.insert(b, -1)
        tb.show_all()
    return tb


class SourcePane(gtk.TreeView):

    def __init__(self, *args, **kwargs):
        super(SourcePane, self).__init__(*args, **kwargs)
        self.tvcolumn = gtk.TreeViewColumn('Source')
        self.set_search_column(0)
        self.append_column(self.tvcolumn)
        self.cell = gtk.CellRendererText()
        self.tvcolumn.pack_start(self.cell, False)
        self.tvcolumn.set_attributes(self.cell, text=0)
        self.tvcolumn.set_clickable(True)
        self.tvcolumn.set_sort_column_id(0)
        self.set_reorderable(False)


class ArtistPane(gtk.TreeView):

    def __init__(self, *args, **kwargs):
        super(ArtistPane, self).__init__(*args, **kwargs)
        self.tvcolumn = gtk.TreeViewColumn('Artist')
        self.set_search_column(0)
        self.append_column(self.tvcolumn)
        self.cell = gtk.CellRendererText()
        self.tvcolumn.pack_start(self.cell, False)
        self.tvcolumn.set_attributes(self.cell, text=0)
        self.tvcolumn.set_clickable(True)
        self.tvcolumn.set_sort_column_id(0)
        self.set_reorderable(True)



class AlbumPane(gtk.TreeView):

    def __init__(self, *args, **kwargs):
        super(AlbumPane, self).__init__(*args, **kwargs)
        self.tvcolumn = gtk.TreeViewColumn('Album')
        self.set_search_column(0)
        self.tvcolumn.set_clickable(True)
        self.append_column(self.tvcolumn)
        self.cell = gtk.CellRendererText()
        self.tvcolumn.pack_start(self.cell, False)
        self.tvcolumn.set_attributes(self.cell, text=0)
        self.set_reorderable(True)
        self.tvcolumn.set_sort_column_id(0)

class SongPane(gtk.TreeView):

    def __init__(self, *args, **kwargs):
        super(SongPane, self).__init__(*args, **kwargs)
        self.rating_column = gtk.TreeViewColumn('rating')
        self.title_column = gtk.TreeViewColumn('Title')
        self.album_column = gtk.TreeViewColumn('Album')
        self.artist_column = gtk.TreeViewColumn('Artist')
        self.set_search_column(0)

        self.rating_column.set_sort_column_id(6)
        self.title_column.set_sort_column_id(2)
        self.album_column.set_sort_column_id(1)
        self.artist_column.set_sort_column_id(0)

        for column in [self.rating_column,
                       self.title_column,
                       self.album_column,
                       self.artist_column]:

            column.set_resizable(True)
            column.set_expand(False)
            self.append_column(column)

        self.rating = gtk.CellRendererText()
        self.title = gtk.CellRendererText()
        self.album = gtk.CellRendererText()
        self.artist = gtk.CellRendererText()

        self.rating_column.pack_start(self.rating, False)
        self.rating_column.set_attributes(self.rating, text=6)

        self.title_column.pack_start(self.title, False)
        self.title_column.set_attributes(self.title, text=2)

        self.album_column.pack_start(self.album, False)
        self.album_column.set_attributes(self.album, text=1)

        self.artist_column.pack_start(self.artist, False)
        self.artist_column.set_attributes(self.artist, text=0)
        self.set_reorderable(True)


class LibraryPane(gtk.TreeView):

    def __init__(self, *args, **kwargs):
        super(LibraryPane, self).__init__(*args, **kwargs)



class MainWindow(gtk.Window):

    def __init__(self, *args, **kwargs):
        super(MainWindow, self).__init__(*args, **kwargs)
        self.set_title("Google Music Player")


class Login(gtk.Dialog):

    def __init__(self, *args, **kwargs):
        super(Login, self).__init__(*args, **kwargs)
        self.label = gtk.Label("set your mail and password in order to connect")
        self.mail = gtk.Entry()
        self.password = gtk.Entry()
        self.password.set_visibility(False)
        self.message = gtk.Dialog()
        self.button = gtk.Button(stock=gtk.STOCK_APPLY)
        self.button.connect("clicked", self.clicked)
        self.vbox.pack_start(self.label)
        self.vbox.pack_start(self.mail)
        self.vbox.pack_start(self.password)
        self.action_area.pack_start(self.button)
        self.show_all()

    def clicked(self, widget):
        user_email = self.mail.get_text()
        user_password = self.password.get_text()
        path = os.path.expanduser("~/.config/google_music_player")
        with open(path, "w") as fd:
            fd.write("{}\n".format(user_email))
            fd.write("{}\n".format(user_password))
        self.destroy()



import urllib
import urllib2
import time
from datetime import datetime
from time import mktime
from md5 import md5


PROTOCOL_VERSION = '1.2'
login_url = "http://post.audioscrobbler.com/"
class AudioScrobbler(object):

    def __init__(self):
        self.cache = []

    def login(self, login, password):
        """
        Log to audioscrobbler services
        """
        tstamp = int(mktime(datetime.now().timetuple()))
        password = md5(password).hexdigest()
        token = md5( "%s%d" % (password, int(tstamp))).hexdigest()
        values = {
            'hs': 'true',
            'p' : PROTOCOL_VERSION,
            'c': 'tst',
            'v': '1.0',
            'u': login,
            't': tstamp,
            'a': token
        }
        data = urllib.urlencode(values)
        req = urllib2.Request("%s?%s" % (login_url, data) )
        response = urllib2.urlopen(req)
        result = [line.strip() for line in response.readlines()]
        if result[0] == "OK":
            self.session = result[1]
            self.now_url = result[2]
            self.post_url = result[3]

    def now_playing(self, artist, album, track, *args):
        """
        add a song to a now playing
        """
        values = {'s': self.session,
                  'a': unicode(artist).encode('utf-8'),
                  't': unicode(track).encode('utf-8'),
                  'b': unicode(album).encode('utf-8'),
                  'l': '',
                  'n': '',
                  'm': ''}

        data = urllib.urlencode(values)
        req = urllib2.Request(self.now_url, data)
        response = urllib2.urlopen(req)
        result = [line.strip() for line in response.readlines()]
        if result[0] == "OK":
            self.now_playing_cache = [artist, album, track]
            return True

    def submit(self, artist, album, track, rating="",
               length="", trackno="", mbid=""):
        """
        submit a song to a cache
        """
        timestamp = time.time()

        values = { 'a': unicode(artist).encode('utf-8'),
                   't': unicode(track).encode('utf-8'),
                   'i': timestamp,
                   'o': "U",
                   'r': rating,
                   'l': length,
                   'b': unicode(album).encode('utf-8'),
                   'n': trackno,
                   'm': mbid
               }
        self.cache.append(values)
        if len(self.cache) > 4:
            self.flush()
    def flush(self):
        """
        flush the cache. post all cached songs to lastfm
        """
        values = {}
        for i, item in enumerate(self.cache):
            for key in item:
                values[key + "[%d]" % i] = item[key]
        values['s'] = self.session
        data = urllib.urlencode(values)
        req = urllib2.Request(self.post_url, data)
        response = urllib2.urlopen(req)
        result = [line.strip() for line in response.readlines()]
        if result[0] == "OK":
            return True



import vlc
import gtk
import urllib2
from gmusicapi.utils import utils
from uuid import getnode as getmac
from vlc import callbackmethod
mac_int = getmac()
if (mac_int >> 40) % 2:
    raise OSError("a valid MAC could not be determined."
                  " Provide an android_id (and be"
                  " sure to provide the same one on future runs).")

android_id = utils.create_mac_string(mac_int)
android_id = android_id.replace(':', '')

@callbackmethod
def SongFinished(self, player):
    if player.lastfm:
        player.lastfm.submit(*player.lastfm.now_playing_cache)
    player.api.increment_song_playcount(
        player.win.liststore[player.win.get_index()][2])
    player.win.set_index(player.win.get_index() + 1)
    player.play()


class MusicPlayer(object):

    def __init__(self, api, win, tree, lastfm=None):

        super(MusicPlayer, self).__init__()
        self.win = win
        self.store = win.liststore
        self.tree = tree
        self.api = api
        self.lastfm = lastfm
        self.p = vlc.MediaPlayer()

    def make_label(self, data):
        artist = data[0]
        album = data[1]
        song = data[2]
        text = "{}-{}-{}".format(
            song,
            album,
            artist)
        if len(text) > 50:
            text = text[:50] + '...'
        self.win.song_label.set_text(
            text
                )
    def _play(self, data):
        if self.p.get_state() in [vlc.State.Playing, vlc.State.Paused] :
            self.p.stop()
        song = self.api.get_stream_url(
            data[3],
            device_id=android_id)
        self.p = vlc.MediaPlayer(song)
        self.p.play()
        self.p.event = self.p.event_manager()
        self.p.event.event_attach(
            vlc.EventType.MediaPlayerEndReached, SongFinished, self)

    def play(self):
        """
        data is : [artist, album, song, uuid, cover]
        """
        selection = self.tree.get_selection()
        selection.select_path(self.win.get_index())
        try:
            data = self.win.liststore[self.win.get_index()]
        except IndexError:
            return
        if self.lastfm:
            self.lastfm.now_playing(*data)
        self._play(data)
        self.make_label(data)
        self.create_thumb()

    def create_thumb(self):
        response=urllib2.urlopen(self.store[self.win.get_index()][4])
        loader=gtk.gdk.PixbufLoader()
        loader.set_size(200,200)
        loader.write(response.read())
        loader.close()
        self.win.album_pic.set_from_pixbuf(loader.get_pixbuf())



import os
from setuptools import setup


def read_relative_file(filename):
    """Returns contents of the given file, which path is supposed relative
    to this module."""
    with open(os.path.join(os.path.dirname(__file__), filename)) as f:
        return f.read().strip()


README = read_relative_file('README.rst')

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))


setup(
    name='googlemusicplayer',
    version='0.5',
    packages=['music_player'],
    include_package_data=True,
    description='a GTK music player using google music API',
    long_description=README,
    author='Yohann Gabory',
    author_email="yohann@gabory.fr",
    LICENCE = "BSD",
    CLASSIFIERS = [
        'Programming Language :: Python :: 2.7',
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: BSD License',
    ],
    install_requires=[
        'gmusicapi',
        'python-vlc',
        ' pyasn1>=0.1.8'
    ],
    scripts=['bin/run'],
)
